#include <cstdlib>
#include <cstring>
#include <sstream>
#include <gst/gst.h>
#include "log.h"
using namespace std;

#define USE(x) ((void)(x))

static GstPipeline *gst_pipeline = nullptr;
static string launch_string;

GstClockTime usec = 1000000;


void my_log_func(GstDebugCategory * category,
                 GstDebugLevel level,
                 const gchar * file,
                 const gchar * function,
                 gint line,
                 GObject * object,
                 GstDebugMessage * message,
                 gpointer user_data) {
    write_log("[GST]%s",gst_debug_message_get(message));
 
}

int main(int argc, char** argv) {
    USE(argc);
    USE(argv);
    log_init();
    gst_debug_add_log_function(my_log_func, NULL, NULL);
    gst_debug_set_active(TRUE);
    gst_debug_set_default_threshold(GST_LEVEL_ERROR);
    gst_init (&argc, &argv);

    GMainLoop *main_loop;
    main_loop = g_main_loop_new (NULL, FALSE);
    ostringstream launch_stream;
    GstBus* bus;
    gboolean  res;

    launch_stream
    << "videotestsrc is-live=1 ! "
    << "video/x-raw,width=1280,height=960,format=YUY2,framerate=30/1 ! "
    << "nvvidconv ! video/x-raw(memory:NVMM),format=I420,width=1280,height=960,framerate=30/1 ! "
    << "nvv4l2h265enc ! fakesink ";

    launch_string = launch_stream.str();

    g_print("Using launch string: %s\n", launch_string.c_str());

    GError *error = nullptr;
    for(guint i=0; i<30000; i++) {
        write_log("loop= %d", i);
        gst_pipeline  = (GstPipeline*) gst_parse_launch(launch_string.c_str(), &error);

        if (gst_pipeline == nullptr) {
            write_log("Failed to parse launch: %s", error->message);
            return -1;
        }
        if(error) 
        {
            write_log("Parse error[%d]",i);
            g_error_free(error);
        }

        write_log("Set playing[%d]",i);
        gst_element_set_state((GstElement*)gst_pipeline, GST_STATE_PLAYING); 
        g_usleep(3*usec);
        write_log("Send Eos[%d]",i);
        res=gst_element_send_event ((GstElement*)gst_pipeline, gst_event_new_eos ());
        if(!res)
        {
            write_log("eos signal send failed\n");
        }
        else
        {
            write_log("eos signal send success"); 
        }
        write_log("get bus[%d]",i);
        bus = gst_pipeline_get_bus(GST_PIPELINE(gst_pipeline));
        write_log("bus poll start[%d]",i);
        gst_bus_poll(bus, GST_MESSAGE_EOS, GST_CLOCK_TIME_NONE);
        write_log("bus poll end &Set NULL[%d]",i);
        gst_element_set_state((GstElement*)gst_pipeline, GST_STATE_NULL);
        write_log("Set unref[%d]",i);
        gst_object_unref(GST_OBJECT(gst_pipeline));
        g_usleep(1*usec);
    }
    g_main_loop_unref(main_loop);

    g_print("going to exit \n");
    return 0;
}
